<?php
/*
   TM Offline Stats v1.2
   Copyright (c) 2006-2010 Alexander Domnin

   This program is licenced under the
   Creative Commons Attribution-Noncommercial-No Derivative Works 3.0 Unported.
   To view a copy of this licence, visit http://creativecommons.org/licenses/by-nc-nd/3.0/

   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

require_once 'tmos_funcs.inc.php';

// *****************************************************************************
// Ttmos_GBXI
// *****************************************************************************

class Ttmos_GBXI {

   /*
      result = array (
                  'b' - uid
                  'c' - name
                  'd' - clear name
                  'e' - author
                  'f' - envir
                  'g' - mood
                  'h' - type = Race
                  'i' - nblaps
                  'j' - price
                  'k' - bronze
                  'l' - silver
                  'm' - gold
                  'n' - author time
                  'o' - author score
                  'r' - filename
                  'x' - version
               )
   */
   function track_xmlheader(&$xmlheader) {
      $vals = array();
      $tags = array();

      // ????
      $xmlheader = str_replace('', '?', $xmlheader);

      $xml_parser = xml_parser_create('UTF-8');
      xml_parser_set_option($xml_parser, XML_OPTION_SKIP_WHITE, 0);
      xml_parse_into_struct($xml_parser, $xmlheader, $vals, $tags);
      xml_parser_free($xml_parser);

      if (isset($tags['HEADER'][0]) &&
          isset($tags['IDENT'][0]) &&
          isset($tags['DESC'][0]) &&
          isset($tags['TIMES'][0]) &&
          isset($vals[$tags['HEADER'][0]]['attributes']['TYPE']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['TYPE']) &&
          ($vals[$tags['HEADER'][0]]['attributes']['TYPE'] == "challenge") &&
          ($vals[$tags['DESC'][0]]['attributes']['TYPE'] == "Race") &&
          isset($vals[$tags['HEADER'][0]]['attributes']['VERSION']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['UID']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['NAME']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['AUTHOR']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['ENVIR']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['MOOD']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['NBLAPS']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['PRICE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['BRONZE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['SILVER']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['GOLD']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['AUTHORSCORE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['AUTHORTIME'])) {

         $result = array();
         $result['b'] = $vals[$tags['IDENT'][0]]['attributes']['UID'];
         $result['c'] = urldecode($vals[$tags['IDENT'][0]]['attributes']['NAME']);
         $result['d'] = name_clear($result['c']);
         $result['e'] = urldecode($vals[$tags['IDENT'][0]]['attributes']['AUTHOR']);
         $result['f'] = $vals[$tags['DESC'][0]]['attributes']['ENVIR'];
         $result['g'] = trim($vals[$tags['DESC'][0]]['attributes']['MOOD']);
         if (preg_match("/^\d{2}x\d{2}(\w+)/i", $result['g'], $matches))
            $result['g'] = ucfirst($matches[1]);
         $result['h'] = $vals[$tags['DESC'][0]]['attributes']['TYPE'];
         $result['i'] = $vals[$tags['DESC'][0]]['attributes']['NBLAPS']; // > 0 = ml
         $result['j'] = $vals[$tags['DESC'][0]]['attributes']['PRICE'];
         $result['k'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['BRONZE']);
         $result['l'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['SILVER']);
         $result['m'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['GOLD']);
         $result['n'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['AUTHORTIME']);
         $result['o'] = $vals[$tags['TIMES'][0]]['attributes']['AUTHORSCORE'];
         if ($result['o'] < 0) $result['o'] = 0;
         $result['r'] = '?'; // filename
         $result['x'] = $vals[$tags['HEADER'][0]]['attributes']['VERSION'];
      }
      else 
         $result = -1;

      return $result;
   }

   // based on http://en.tm-wiki.org/wiki/GBX
   function track_hexheader(&$hexheader) {
      // check signature and gbx type
      if (((substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCR') &&
           (substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCE')) ||
          ((substr($hexheader, 9, 4) != "\x00\x30\x00\x24") &&
           (substr($hexheader, 9, 4) != "\x00\x30\x04\x03"))) {
         return -1; // no challenge
      }
      $challver = str_toint(substr($hexheader, 17, 4)); // challenge version
      // times/info block ...
      if ($challver >= 5)
         $offset = 61;
      else if ($challver >= 4)
         $offset = 53;
      else if ($challver >= 3)
         $offset = 45;
      else
         $offset = 37;
      $exever = str_toint($hexheader[$offset]); // exe version
      if ($exever >= 6 && str_toint(substr($hexheader, $offset + 29, 4)) != 0) {
         return -1; // no race
      }
      $result = array();
      $result['k'] = time_msectofull(str_toint(substr($hexheader, $offset  + 5, 4))); // bronze
      $result['l'] = time_msectofull(str_toint(substr($hexheader, $offset  + 9, 4))); // silver
      $result['m'] = time_msectofull(str_toint(substr($hexheader, $offset + 13, 4))); // gold
      $result['n'] = time_msectofull(str_toint(substr($hexheader, $offset + 17, 4))); // author
      if ($challver >= 3)
         $result['j'] = str_toint(substr($hexheader, $offset + 21, 4)); // price
      else
         $result['j'] = 0;
      if ($challver >= 3 && $exever >= 6) {
         $result['i'] = str_toint(substr($hexheader, $offset + 25, 4)); // nblaps > 0 = ml
         $result['h'] = str_toint(substr($hexheader, $offset + 29, 4)); // type
         if ($result['h'] == 0)
            $result['h'] = 'Race';
         else if ($result['h'] == 1)
            $result['h'] = 'Platform';
         else if ($result['h'] == 2)
            $result['h'] = 'Puzzle';
         else if ($result['h'] == 3)
            $result['h'] = 'Crazy';
         else if ($result['h'] == 4)
            $result['h'] = 'Stunts';
      }
      else {
         $result['i'] = 0;
         $result['h'] = '';
      }
      if ($challver >= 4)
         $result['o'] = str_toint(substr($hexheader, $offset + 37, 4)); // authorscore
      else
         $result['o'] = 0;
      // strings block ...
      $offset += str_toint(substr($hexheader, 25, 4));
      if ($challver >= 3)
         $offset += 9;
      else
         $offset += 5;
      $len = str_toint(substr($hexheader, $offset, 4)); // uid len
      $result['b'] = substr($hexheader, $offset + 4, $len); // uid
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // envir len
      $result['f'] = substr($hexheader, $offset + 4, $len); // envir
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // author len
      $result['e'] = substr($hexheader, $offset + 4, $len); // author
      if (substr($result['e'], 0, 3) == "\xEF\xBB\xBF")  // del BOM
         $result['e'] = substr($result['e'], 3);
      $offset += 4 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // track len
      $result['c'] = substr($hexheader, $offset + 4, $len); // track
      if (substr($result['c'], 0, 3) == "\xEF\xBB\xBF") // del BOM
         $result['c'] = substr($result['c'], 3);
      $result['d'] = name_clear($result['c']);
      if ($challver >= 4) {
         $offset += 9 + $len;
         $len = str_toint(substr($hexheader, $offset, 4)); // passw len
         $offset += 8 + $len;
         $len = str_toint(substr($hexheader, $offset, 4)); // mood len
         $result['g'] = substr($hexheader, $offset + 4, $len); // mood
         if (preg_match("/^\d{2}x\d{2}(\w+)/i", $result['g'], $matches))
            $result['g'] = ucfirst($matches[1]);
      }
      else
         $result['g'] = '';
      // other
      $result['r'] = '?'; // filename
      $result['x'] = $challver;

      return $result;
   }

   function cmp($a, $b) {
      return strcasecmp($a['d'], $b['d']);
   }

   /*
      result = array('#'=>array('track info'))
   */
   function tracks_info($dir, $sort = false) {
      $result = array();

      $list = filelist_ex($dir, '.*\.gbx');
      foreach ($list as $filename) {
         // xml header
         $buf = @filedscr_ex($filename, '<header', '/header>', false);
         if ($buf != -1) {
            $buf = $this->track_xmlheader($buf);
         }
         // hex header
         if ($buf == -1) {
            $buf = @filedscr_ex($filename, 0, 2047, false);
            if ($buf != -1) {
               $buf = $this->track_hexheader($buf);
            }
         }
         if ($buf !== -1) {
            $buf['r'] = $filename;
            $result[] = $buf;
         }
      }
      if ($sort) uasort($result, array("Ttmos_GBXI", "cmp"));

      return $result;
   }

   /*
      result = array (
                  'b' - uid
                  'c' - thumb string
               )
   */
   function thumb_strheader(&$strheader) {
      $offset_uid = strpos($strheader, 'uid="');
      $offset_tn = strpos($strheader, '<Thumbnail.jpg>');

      if ($offset_uid === false || $offset_tn === false)
         $result = -1;
      else {
         $result = array('b'=>substr($strheader, $offset_uid + 5, 
                                  strpos($strheader, '" ', $offset_uid + 6) - $offset_uid - 5),
                         'c'=>substr($strheader, $offset_tn + 15,
                                 strpos($strheader, '</Thumbnail.jpg>') - $offset_tn - 15));
         // very big image
         if ($result['b'] == '' || $result['c'] == '')
            $result = -1;
      }

      return $result;
   }

   /*
      result = array('#'=>array('thumb info'))
   */
   function thumbs_info($dir) {
      $result = array();

      $list = @filelist_ex($dir, '.*\.gbx');
      foreach ($list as $filename) {
         $buf = @filedscr_ex($filename, '<header', '</Thumbnail.jpg>', false, 32768);
         if ($buf != -1) {
            $buf = $this->thumb_strheader($buf);
            if ($buf != -1)
               $result[] = $buf;
         }
      }

      return $result;
   }

   /*
      result = array (
                  'b' - uid
                  'c' - envir
                  'd' - author
                  'e' - login/account
                  'f' - player
                  'g' - result time
               )
   */
   function replay_hexheader(&$hexheader) {
      $result = array();

      // check signature and gbx type
      if (((substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCR') &&
           (substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCE')) ||
          ((substr($hexheader, 9, 4) != "\x00\xE0\x07\x24") &&
           (substr($hexheader, 9, 4) != "\x00\xF0\x03\x24") &&
           (substr($hexheader, 9, 4) != "\x00\x30\x09\x03"))) {
         return -1; // no replay
      }
      $repver = str_toint(substr($hexheader, 17, 4)); // replay version
      if ($repver < 2) {
         return -1; // no login/account
      }
      $exever = str_toint(substr($hexheader, 37, 4)); // exe version
      if ($exever < 6) {
         return -1; // no login/account
      }
      $offset = 49;
      $len = str_toint(substr($hexheader, $offset, 4)); // uid len
      $result['b'] = substr($hexheader, $offset + 4, $len); // uid
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // envir len
      $result['c'] = substr($hexheader, $offset + 4, $len); // envir
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // author len
      $result['d'] = substr($hexheader, $offset + 4, $len); // author
      if (substr($result['d'], 0, 3) == "\xEF\xBB\xBF") // del BOM
         $result['d'] = substr($result['d'], 3);
      $offset += 4 + $len;
      $buf = substr($hexheader, $offset, 4); // time
      if ($buf == "\xFF\xFF\xFF\xFF") {
         return -1; // lan replay
      }
      $result['g'] = time_msectofull(str_toint($buf));
      $offset += 4;
      $len = str_toint(substr($hexheader, $offset, 4)); // player len
      $result['f'] = substr($hexheader, $offset + 4, $len); // player
      if (substr($result['f'], 0, 3) == "\xEF\xBB\xBF") // del BOM
         $result['f'] = substr($result['f'], 3);
      $offset += 4 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // login len
      $result['e'] = substr($hexheader, $offset + 4, $len); // login

      return $result;
   }

   /*
      result = array('#'=>array('replay info'))
   */
   function replays_info($dir) {
      $result = array();

      $list = @filelist_ex($dir, '.*\.gbx');
      foreach ($list as $filename) {
         $buf = @filedscr_ex($filename, 0, 1023, false, 1024);
         if ($buf != -1) {
            $buf = $this->replay_hexheader($buf);
            if ($buf != -1)
               $result[] = $buf;
         }
      }

      return $result;
   }

   /*
      result = array (
                  'b' - uid
                  'c' - filename
               )
   */
   function track_strheader(&$strheader, $filename) {
      $offset = strpos($strheader, 'uid="');
      if ($offset === false)
         $result = -1;
      else {
         $result = array ('b'=>substr($strheader, $offset + 5, 
                                  strpos($strheader, '" ', $offset + 6) - $offset - 5),
                          'c'=>$filename);
      }
      return $result;
   }

   /*
      result = array('#'=>array('uid', 'filename'))
   */
   function tracks_list($dir) {
      $result = array();

      $list = @filelist_ex($dir, '.*\.gbx');
      foreach ($list as $filename) {
         $buf = @filedscr_ex($filename, '<header', '/header>', false);
         if ($buf != -1) {
            $buf = $this->track_strheader($buf, $filename);
            if ($buf != -1)
               $result[] = $buf;
         }
      }

      return $result;
   }

} // class

?>